# Copyright(c) 1998, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: IconBar.tcl,v 1.24 2005/04/14 19:41:20 tsmith Exp $

#------------------------------------------------------------------------------#
# InitIconBar - initial entry point (called in GUIMain.gui)                    #
#------------------------------------------------------------------------------#
proc InitIconBar {} \
{
   global MCMacros
   global MCMacroNum
   global MCMacrosToShow
   global MCCount
   global MCFirst
   global mainIconFrame
   global popup_help_text

   # Display buttons starting with the first one
   set MCFirst 0

   # The button frame is not packed here, the packing/unpacking is handled
   # in initmb.tcl under the 'View' menu.
   frame .userDefinedButtonFrame \
	-relief groove \
	-borderwidth 2

   set IconFrame .userDefinedButtonFrame
   set mainIconFrame $IconFrame

   if { [info exists MCMacroNum] } { MCDrawButtons }
}

#------------------------------------------------------------------------------#
# MCShowPrev -- Shows the previous set of buttons                              #
#------------------------------------------------------------------------------#
proc MCShowPrev { } \
{
  global MCMacros MCMacroNum
  global MCCount MCFirst
  global mainIconFrame

  set IconFrame $mainIconFrame
  catch { destroy $IconFrame.fmButtons }
  catch { destroy $IconFrame.btNext }

  set MCFirst [expr $MCFirst - $MCCount]
  if { $MCFirst < 0 } { set MCFirst [expr $MCMacroNum - $MCMacroNum % $MCCount] }
  if { $MCFirst >= $MCMacroNum } { set MCFirst [expr $MCMacroNum - $MCCount] }
  if { $MCFirst < 0} { set MCFirst 0 }

  MCDrawButtons
}
  
#------------------------------------------------------------------------------#
# MCShowNext -- Shows the next set of buttons                                  #
#------------------------------------------------------------------------------#
proc MCShowNext { } \
{
  global MCMacros MCMacroNum
  global MCCount MCFirst
  global mainIconFrame

  set IconFrame $mainIconFrame
  catch { destroy $IconFrame.fmButtons }
  catch { destroy $IconFrame.btNext }

  set MCFirst [expr $MCFirst + $MCCount]
  if { $MCFirst >= $MCMacroNum } { set MCFirst 0 }
  
  MCDrawButtons
}


#------------------------------------------------------------------------------#
# MCDrawButtons -- Draws the button set -- Macros list                         #
#------------------------------------------------------------------------------#
proc MCDrawButtons { } \
{
  global MCMacros MCMacroNum
  global MCCount MCFirst
  global mainIconFrame
  global popup_help_text
  
  set IconFrame $mainIconFrame
  catch { destroy $IconFrame.btPrev }
  catch { destroy $IconFrame.fmButtons }
  catch { destroy $IconFrame.btNext }  
  
   # 'Prev' button (only show if necessary)
   if { $MCMacroNum <= $MCCount } \
   {
	 catch { destroy $IconFrame.btPrev }
   } \
   else \
   {
	button $IconFrame.btPrev \
	    -text "Prev" \
	    -command { MCShowPrev }
	pack $IconFrame.btPrev \
	    -side left \
	    -fill y	
	set popup_help_text($IconFrame.btPrev) "Show previous set of buttons"
   }

   # User-configured buttons (macros)
   if { [info exists MCMacroNum] } \
   {
	frame $IconFrame.fmButtons
	pack $IconFrame.fmButtons \
	    -side left \
	    -fill x \
	    -expand yes

	# Set last button to draw
	set UpTo [expr $MCFirst + $MCCount]
	if { $UpTo > $MCMacroNum } { set UpTo $MCMacroNum }

	# Let's draw some buttons!
	for { set i $MCFirst } { $i < $UpTo } { incr i } \
	{	
	   if { [string match $MCMacros($i,T) "BUTTON"] } \
	   {
	   	button $IconFrame.fmButtons.bt$i \
		   -text $MCMacros($i,N) \
		   -command "MCDoCommand $i -1" \
		   -width 1
            	pack $IconFrame.fmButtons.bt$i \
		   -side left \
		   -fill x \
		   -expand yes
	   } \
	   elseif { [string match "MENU" $MCMacros($i,T)] } \
	   {
		menubutton $IconFrame.fmButtons.bt$i \
		   -text $MCMacros($i,N) \
		   -indicatoron YES \
		   -relief raised \
		   -borderwidth 2 \
		   -width 1 \
		   -menu $IconFrame.fmButtons.bt$i.menu
		pack $IconFrame.fmButtons.bt$i \
		   -side left \
		   -fill x \
		   -expand yes
		menu $IconFrame.fmButtons.bt$i.menu \
		   -tearoff NO
		
		# Add menu items
		set num_menu_items [lindex $MCMacros($i,C) 0]
		for { set j 0 } { $j < $num_menu_items } { incr j } \
		{
		    $IconFrame.fmButtons.bt$i.menu add command \
		    	-label [MCParseMenuItem $MCMacros($i,M) $j "NAME"] \
			-command "MCDoCommand $i $j"
		}
	   }

	   # Add popup help text only if the string length is greater than zero
	   if { [string length [string trim $MCMacros($i,P)]] > 0 } \
	   {
		set popup_help_text($IconFrame.fmButtons.bt$i) $MCMacros($i,P)
	   }
	}

	# If the number of buttons to draw is greater than the number of
	# buttons that have been configured then draw the rest of the
	# buttons with state=disabled.
    	for { set i $UpTo } { $i < [expr $MCFirst + $MCCount] } { incr i } \
    	{
	   button $IconFrame.fmButtons.bt$i \
		-text "" \
		-state disabled \
		-width 1
	   pack $IconFrame.fmButtons.bt$i \
		-side left \
		-fill x \
		-expand yes
    	}
   }

   # 'Next' button (only show if necessary)
   if { $MCMacroNum <= $MCCount } \
   {
	catch { destroy $IconFrame.btNext }
   } \
   else \
   {
	button $IconFrame.btNext \
	    -text "Next" \
     	    -command { MCShowNext }
     	pack $IconFrame.btNext \
	    -side left \
	    -fill y
	set popup_help_text($IconFrame.btNext) "Show next set of buttons"
   }
}


#------------------------------------------------------------------------------#
# MCDoCommand -- Executes a command                                            #
#------------------------------------------------------------------------------#
proc MCDoCommand { line menu_item } \
{
  global MCMacros
  
  # If 'menu_item' is -1 then 'MCDoCommand' was called by a button, if 'menu_item'
  # is >= 0 then 'MCDoCommand' was called by a menu button for menu item 'menu_item'.
  if { $menu_item < 0 } \
  {
  	set KeyIn $MCMacros($line,C)
  } \
  else \
  {
	set KeyIn [MCParseMenuItem $MCMacros($line,M) $menu_item "COMMAND"]
  }
  
  # Look for semicolons separating multiple commands and execute each
  # command seperately.
  while { [string first ";" $KeyIn] != -1 } \
  {
    	set i [string first ";" $KeyIn]
    	set ToExec [string range $KeyIn 0 [expr $i -1]]
    	set KeyIn [string range $KeyIn [expr $i + 1] end ]
    	catch { [RunCommand "$ToExec"] } status
 	if {[string compare $status "program not found"] == 0} \
  	{
	    tk_messageBox \
		-message "$ToExec is not a valid McIDAS command.\
		          Click on \'Shortcut Buttons\' under the \'Configure\' menu\
		          to correct this error." \
	   	-type ok \
	   	-icon error \
	   	-title " Error"
	    showIdleStatus
  	}
  }
  
  # Execute the last command in the string (or the only command if there
  # was just one). 
  catch { [RunCommand "$KeyIn"] } status
  if {[string compare $status "program not found"] == 0} \
  {
	tk_messageBox \
	   -message "$KeyIn is not a valid McIDAS command.\
		Click on \'Shortcut Buttons\' under the \'Configure\' menu\
		to correct this error." \
	   -type ok \
	   -icon error \
	   -title " Error"
	showIdleStatus
  }
}


#------------------------------------------------------------------------------#
# MCMacroBuilder  --  Lists and allows editing of IconBar Macro definitions    #
#------------------------------------------------------------------------------#
proc MCMacroBuilder { } \
{
  #------------------------------------------------------#
  # MCMacros(#,N) -- Name
  # MCMacros(#,C) -- Command string
  # MCMacros(#,T) -- Type (BUTTON, MENU, or MENUITEM)
  
  global MCMacros
  global MCMacroNum
  global MCMacroList
  global MCMacroVisible
  global MCMacrosEdit
  global MCMacroNumEdit
  global MCFirst
  global MCCount
  global MCCountOld
  
  # Keep track of old MCCount value in case the user
  # changes the value but then hits 'Dismiss'.
  set MCCountOld $MCCount

  if { ![info exists MCMacroNum] } { set MCMacroNum 0 }

  # Save all macro information from 'MCMacros' to the temporary
  # array (MCMacrosEdit)
  MCEditMacros
  
  if { [info exists MCMacroVisible] && $MCMacroVisible==1 } \
  {
    wm deiconify .tlMacro
    focus .tlMacro
    raise .tlMacro
    return
  }
  set MCMacroVisible 1
  
  toplevel .tlMacro 
  wm title .tlMacro "Shortcut Buttons"
  wm resizable .tlMacro no no
  wm geometry .tlMacro +100+100
  
  frame .tlMacro.fmTop \
  	-relief raised \
	-borderwidth 2
  pack .tlMacro.fmTop \
  	-side top \
	-padx 2 \
	-pady 2
		
  frame .tlMacro.fmTop.fmList \
  	-relief raised \
	-borderwidth 2
  pack .tlMacro.fmTop.fmList \
    	-side top \
	-expand true \
	-fill both \
	-padx 5 \
	-pady 5
    
  set MCMacroList [MakeListBox .tlMacro.fmTop.fmList \
                   [format "%-12s %-12s %s" "Name" "Type" "Command"] ""]
  for { set i 0 } { $i < $MCMacroNum } { incr i } \
  {
    if { [string match $MCMacros($i,T) "MENU"] } \
    {
	    set command_string ""
    } \
    else \
    {
	    set command_string $MCMacros($i,C)
    }
    
    set toList [format "%-15s %-15s %s" [string range $MCMacros($i,N) 0 11] \
	                                $MCMacros($i,T) \
	                                $command_string]
    $MCMacroList insert $i $toList
  }
  
  # Number of buttons to show
  frame .tlMacro.fmTop.fmNumToShow
  pack .tlMacro.fmTop.fmNumToShow \
  	-side top \
	-padx 5 \
	-pady 5
	
  label .tlMacro.fmTop.fmNumToShow.label \
  	-text "Number of buttons to show: "
  pack .tlMacro.fmTop.fmNumToShow.label \
  	-side left
	
  spinbox .tlMacro.fmTop.fmNumToShow.spinBox \
  	-from 1 \
	-to 10 \
	-width 2 \
	-textvariable MCCount \
	-state readonly \
	-disabledbackground white
  pack .tlMacro.fmTop.fmNumToShow.spinBox
      
  # Editing controls
  frame .tlMacro.fmTop.fmOptions
  pack .tlMacro.fmTop.fmOptions \
  	-side top \
	-padx 5 \
	-pady 5 

  button .tlMacro.fmTop.fmOptions.btNew \
      	-text "New Button" \
	-width 16 \
	-command "MCNewMacro BUTTON"
  pack .tlMacro.fmTop.fmOptions.btNew \
  	-side left \
	-padx 5 \
	-pady 5
      
  button .tlMacro.fmTop.fmOptions.btEdit \
      	-text "Edit" \
	-width 8 \
	-command { MCMacroEdit [MCGetLine] }
  pack .tlMacro.fmTop.fmOptions.btEdit \
  	-side left \
	-padx 5 \
	-pady 5

  button .tlMacro.fmTop.fmOptions.btDelete \
      	-text "Delete" \
	-width 8 \
	-command { MCDeleteMacro [MCGetLine] }
  pack .tlMacro.fmTop.fmOptions.btDelete \
  	-side left \
	-padx 5 \
	-pady 5

  button .tlMacro.fmTop.fmOptions.btUp \
      	-text "Up" \
	-width 4 \
	-command { MCMoveMacro [MCGetLine] "UP" }
  pack .tlMacro.fmTop.fmOptions.btUp \
  	-side right \
	-pady 5

  button .tlMacro.fmTop.fmOptions.btDown \
      	-text "Down" \
	-width 4 \
	-command { MCMoveMacro [MCGetLine] "DOWN" }
  pack .tlMacro.fmTop.fmOptions.btDown \
  	-side right \
	-pady 5

  # 'OK' and 'Dismiss' buttons
  frame .tlMacro.fmBottom
  pack .tlMacro.fmBottom \
  	-side top

  button .tlMacro.fmBottom.btOkay \
	-text "OK" \
	-width 12 \
    -command { set MCMacroVisible 0
	       set MCFirst 0
	       MCSaveMacros
	       MCDrawButtons
	       SaveContext
	       destroy .tlMacro }
  pack .tlMacro.fmBottom.btOkay \
  	-side left \
	-padx 5 \
	-pady 5
		   
  button .tlMacro.fmBottom.btExit \
	-text "Dismiss" \
	-width 12 \
	-command { set MCCount $MCCountOld
	           catch { destroy .tlMacroEdit }
	           destroy .tlMacro
	           set MCMacroVisible 0 }
  pack .tlMacro.fmBottom.btExit \
 	-side left \
	-padx 5 \
	-pady 5
	
  # Set double-click behavior for list box
  bind $MCMacroList <Double-1> \
  {
	set line [$MCMacroList curselection]
	MCMacroEdit $line
  }
}    


#------------------------------------------------------------------------------#
# MCNewMacro --  Builds a new Macro location to place data into                #
#                type = BUTTON or MENU                                         #
#------------------------------------------------------------------------------#
proc MCNewMacro { type } \
{
  global MCMacros
  global MCMacroList
  global MCMacrosEdit
  global MCMacroNumEdit
  global ListBoxSelections
  
  # This negates the list-box selection memory
  set ListBoxSelections($MCMacroList) ""
  
  # T=type, N=name, C=command, P=popup message, M=menu list
  set MCMacrosEdit($MCMacroNumEdit,T) $type
  set MCMacrosEdit($MCMacroNumEdit,N) ""
  set MCMacrosEdit($MCMacroNumEdit,C) ""
  set MCMacrosEdit($MCMacroNumEdit,P) ""
  set MCMacrosEdit($MCMacroNumEdit,M) [list]
    
  incr MCMacroNumEdit

  if { [MCMacroEdit [expr $MCMacroNumEdit - 1]] == 0 } \
  {
    # cancel pressed
    incr MCMacroNumEdit -1
  }  
}  


#------------------------------------------------------------------------------#
# MCMacroEdit --  Edit the Macro in the array based on the passed index        #
#------------------------------------------------------------------------------#
proc MCMacroEdit { line } \
{
  global MCMacros
  global MCMacroList
  global MCMacrosEdit
  global MCMacroNumEdit
  global MCNameText
  global MCCommandText
  global MCTypeText
  global MCPopupText
  global MCMenuList
  global MCRetVal
  global line_temp
  global ListBoxSelections
  global MCForceCommandToUpper

  set MCRetVal 0

  # This variable is necessary so that the value of $line can be
  # used within a button's command definition
  set line_temp $line

  # Return if no macro has been selected
  if { $line == -1 || [string length [string trim $line]] == 0 } { return }

  # This negates the list-box selection memory
  set ListBoxSelections($MCMacroList) ""

  set MCNameText    $MCMacrosEdit($line,N)
  set MCCommandText $MCMacrosEdit($line,C)
  set MCTypeText    $MCMacrosEdit($line,T)
  set MCPopupText   $MCMacrosEdit($line,P)
  set MCMenuList    $MCMacrosEdit($line,M)
  
  if { [winfo exists .tlMacroEdit] } \
  {
    wm deiconify .tlMacroEdit
    focus .tlMacroEdit
    raise .tlMacroEdit
    incr MCMacroNumEdit -1
    incr line -1
    incr line_temp -1
    return
  }
  
  toplevel .tlMacroEdit
  wm title .tlMacroEdit "Button Editor"
  wm resizable .tlMacroEdit no no
  wm geometry .tlMacroEdit +250+250

  # Create the macro editing window
  frame .tlMacroEdit.fmEdit \
  	-relief raised \
	-borderwidth 2
  pack .tlMacroEdit.fmEdit \
  	-side top \
	-expand yes \
	-fill x \
	-padx 5 \
	-pady 5 \
	-ipadx 10 \
	-ipady 10
  labelframe .tlMacroEdit.fmEdit.type \
  	-text "Button Type"
  grid .tlMacroEdit.fmEdit.type \
  	-row 0 \
	-column 0 \
	-columnspan 2 \
	-ipady 5 \
	-sticky ew
  radiobutton .tlMacroEdit.fmEdit.type.rbButton \
  	-text "Command button" \
	-variable MCTypeText \
	-value "BUTTON" \
	-highlightthickness 0 \
	-command { SetButtonType BUTTON }
  grid .tlMacroEdit.fmEdit.type.rbButton \
  	-row 0 \
	-column 0 \
	-sticky w
  radiobutton .tlMacroEdit.fmEdit.type.rbMenuButton \
  	-text "Drop-down menu button" \
	-variable MCTypeText \
	-value "MENU" \
	-highlightthickness 0 \
	-command { SetButtonType MENU }
  grid .tlMacroEdit.fmEdit.type.rbMenuButton \
  	-row 0 \
	-column 1 \
	-padx 10 \
	-sticky w
  label .tlMacroEdit.fmEdit.lbBName \
    	-text "Label:"
  grid .tlMacroEdit.fmEdit.lbBName \
    	-row 1 \
	-column 0 \
	-sticky w \
	-pady 10
  entry .tlMacroEdit.fmEdit.enBNameText \
    	-width 15 \
	-textvariable MCNameText \
	-background white
  grid .tlMacroEdit.fmEdit.enBNameText \
    	-row 1 \
	-column 1 \
	-sticky w
  label .tlMacroEdit.fmEdit.lbPopup \
  	-text "Popup Text:"
  grid .tlMacroEdit.fmEdit.lbPopup \
  	-row 2 \
	-column 0 \
	-sticky w
  entry .tlMacroEdit.fmEdit.enPopup \
	-textvariable MCPopupText \
	-background white \
    	-width 40
  grid .tlMacroEdit.fmEdit.enPopup \
  	-row 2 \
	-column 1 \
	-sticky ew
		
  # The following controls will be displayed depending
  # on the type of button that has been selected
  label .tlMacroEdit.fmEdit.lbBCommand \
    	-text "Command:"
  entry .tlMacroEdit.fmEdit.enBCommandText \
	-textvariable MCCommandText \
	-background white
  checkbutton .tlMacroEdit.fmEdit.btForceToUpper \
  	-text "Force command string to upper case" \
	-highlightthickness 0 \
	-variable MCForceCommandToUpper \
	-onvalue "YES" \
	-offvalue "NO"
  button .tlMacroEdit.fmEdit.btEditMenu \
  	-text "Edit Menu Items" \
	-command \
	 {
	    # Just pop up a warning if MCNameText is empty
   	    if { [string length [string trim $MCNameText]] == 0 } \
   	    {
		tk_messageBox \
	    	    -title Error \
	    	    -type ok \
		    -icon error \
	    	    -parent .tlMacroEdit \
	    	    -message "The Label field is mandatory."
   	    } \
	    else \
	    {	
		SaveButtonSettings $line_temp $MCForceCommandToUpper
		MCMenuBuilder $line_temp
	    }
	 }
	
  # Set the selection status of the 'button type' radio buttons
  SetButtonType $MCTypeText
  
  # Disable the 'button type' radio buttons if we are
  # editing an existing button
  if { [string length [string trim $MCNameText]] > 0 } \
  {
	.tlMacroEdit.fmEdit.type.rbButton configure \
	   -state disabled
	.tlMacroEdit.fmEdit.type.rbMenuButton configure \
	   -state disabled
  }

  frame .tlMacroEdit.fmExit
  pack .tlMacroEdit.fmExit \
	-side top \
	-pady 5

  button .tlMacroEdit.fmExit.btOK \
	-text "OK" \
	-width 12 \
	-command \
	 { 
	    # Just pop up a warning if either MCNameText or MCCommandText is empty
   	    if { [string length [string trim $MCNameText]] == 0 } \
   	    {
		tk_messageBox \
	    	    -title Error \
	    	    -type ok \
		    -icon error \
	    	    -parent .tlMacroEdit \
	    	    -message "The Label field is mandatory."
   	    } \
	    elseif { [string match $MCTypeText "BUTTON"] && \
		     [string length [string trim $MCCommandText]] == 0 } \
	    {
	    	tk_messageBox \
	    	    -title Error \
	    	    -type ok \
		    -icon error \
	    	    -parent .tlMacroEdit \
	    	    -message "The Command field is mandatory."
	    } \
	    else \
	    {	
		SaveButtonSettings $line_temp $MCForceCommandToUpper
		set MCRetVal 1
		destroy .tlMacroEdit
	    }
	 }

  pack .tlMacroEdit.fmExit.btOK \
  	-side left \
	-padx 10
    
  button .tlMacroEdit.fmExit.btCancel \
    	-text "Cancel" \
	-width 12 \
	-command { destroy .tlMacroEdit
	           set MCRetVal 0 }
  pack .tlMacroEdit.fmExit.btCancel \
    	-side left \
	-padx 10	 
	 
  tkwait window .tlMacroEdit
  return $MCRetVal
}


#------------------------------------------------------------------------------#
# MCMenuBuilder  --  List and allow editing of IconBar Menu definitions        #
#------------------------------------------------------------------------------#
proc MCMenuBuilder { line } \
{
  global MCMacros
  global MCMacroList
  global MCMenuList
  global macro_line
  
  set macro_line $line

  if { [winfo exists .tlMenu] } \
  {
    wm deiconify .tlMenu
    focus .tlMenu
    raise .tlMenu
    return
  }
  
  toplevel .tlMenu 
  wm title .tlMenu "Menu Editor"
  wm resizable .tlMenu no no
  wm geometry .tlMenu +300+300
  
  frame .tlMenu.fmTop \
  	-relief raised \
	-borderwidth 2
  pack .tlMenu.fmTop \
  	-side top \
	-padx 2 \
	-pady 2
		
  frame .tlMenu.fmTop.fmList \
  	-relief raised \
	-borderwidth 2
  pack .tlMenu.fmTop.fmList \
    	-side top \
	-expand true \
	-fill both \
	-padx 5 \
	-pady 5
  
  # Create the menu item list box  
  set MCMenuList [MakeListBox .tlMenu.fmTop.fmList \
                              [format "%-18s %s" "Menu Item Name"  "Command"] \
			      ""]
			       
  # Populate the menu list box
  MCPopulateMenuList $macro_line
      
  frame .tlMenu.fmTop.fmOptions
  pack .tlMenu.fmTop.fmOptions \
  	-side top \
	-padx 5 \
	-pady 5 

  button .tlMenu.fmTop.fmOptions.btNew \
      	-text "Add New Menu Item" \
	-width 16 \
	-command "MCMenuEdit $macro_line -999"
  pack .tlMenu.fmTop.fmOptions.btNew \
  	-side left \
	-padx 5 \
	-pady 5
      
  button .tlMenu.fmTop.fmOptions.btEdit \
      	-text "Edit" \
	-width 8 \
	-command { MCMenuEdit $macro_line [MCGetMenuLine] }
  pack .tlMenu.fmTop.fmOptions.btEdit \
  	-side left \
	-padx 5 \
	-pady 5

  button .tlMenu.fmTop.fmOptions.btDelete \
      	-text "Delete" \
	-width 8 \
	-command { MCDeleteMenuItem $macro_line [MCGetMenuLine] }
  pack .tlMenu.fmTop.fmOptions.btDelete \
  	-side left \
	-padx 5 \
	-pady 5

  button .tlMenu.fmTop.fmOptions.btUp \
      	-text "Up" \
	-width 4 \
	-command { MCMoveMenuItem "UP" $macro_line [MCGetMenuLine] }
  pack .tlMenu.fmTop.fmOptions.btUp \
  	-side right \
	-pady 5

  button .tlMenu.fmTop.fmOptions.btDown \
      	-text "Down" \
	-width 4 \
	-command { MCMoveMenuItem "DOWN" $macro_line [MCGetMenuLine] }
  pack .tlMenu.fmTop.fmOptions.btDown \
  	-side right \
	-pady 5

  frame .tlMenu.fmBottom
  pack .tlMenu.fmBottom \
  	-side top
	
  button .tlMenu.fmBottom.btOkay \
 	-text "OK" \
	-width 12 \
      	-command { MCSaveMacros
	           MCDrawButtons
	           destroy .tlMenu }
  pack .tlMenu.fmBottom.btOkay \
  	-side left \
	-padx 5 \
	-pady 5

 button .tlMenu.fmBottom.btExit \
	-text "Dismiss" \
	-width 12 \
	-command { destroy .tlMenu }
 pack .tlMenu.fmBottom.btExit \
 	-side left \
	-padx 5 \
	-pady 5
	
  #--- configure the LOOP list box
  bind $MCMenuList <Double-1> { MCMenuEdit $macro_line [$MCMenuList curselection] }
}

#------------------------------------------------------------------------------#
# MCDeleteMenuItem $macro_line [MCGetMenuLine] 
#------------------------------------------------------------------------------#
proc MCDeleteMenuItem { macro_line menu_index } \
{
   global MCMacrosEdit

   set MCMacrosEdit($macro_line,M) [lreplace $MCMacrosEdit($macro_line,M) \
	                                     $menu_index \
	                                     $menu_index]

   # Repopulate the list box
   MCPopulateMenuList $macro_line
}


#------------------------------------------------------------------------------#
# MCMenuEdit --  Edit the menu item in the list based on the passed index.     #
#                A value of -999 for 'menu_index' indicates that this is a new #
#                menu item.                                                    #
#------------------------------------------------------------------------------#
proc MCMenuEdit { line index } \
{
  global MCMacros
  global MCMacroList
  global MCMacrosEdit
  global MCMacroNumEdit
  global MCNameText
  global MCCommandText
  global MCTypeText
  global MCPopupText
  global MCMenuList
  global MCRetVal
  global line_temp
  global ListBoxSelections
  global menu_item_name
  global menu_item_command
  global macro_line
  global menu_index
  global MCForceCommandToUpper

  # This variable is necessary so that the value of 'line' can be
  # used within a button's command definition
  set macro_line $line
  set menu_index $index

  # Return if no menu item has been selected
  if { $menu_index == -1 } { return }

  # Return if there is no item in the list box
  if { [string length [string trim $index]] == 0 } { return } 

  # Get values for the menu item's name and command string
  set menu_item_name    "[MCParseMenuItem $MCMacrosEdit($line,M) \
                                          $menu_index \
					  "NAME"]"
  set menu_item_command "[MCParseMenuItem $MCMacrosEdit($line,M) \
                                          $menu_index \
					  "COMMAND"]"

  if { [winfo exists .tlMenuEdit] } \
  {
    wm deiconify .tlMenuEdit
    focus .tlMenuEdit
    raise .tlMenuEdit
    return
  }

  toplevel .tlMenuEdit
  wm title .tlMenuEdit "Menu Item Editor"
  wm resizable .tlMenuEdit no no
  wm geometry .tlMenuEdit +350+350

  # Create the macro editing window
  frame .tlMenuEdit.fmEdit \
  	-relief raised \
	-borderwidth 2
  pack .tlMenuEdit.fmEdit \
  	-side top \
	-expand yes \
	-fill x \
	-padx 5 \
	-pady 5 \
	-ipadx 10 \
	-ipady 10
  label .tlMenuEdit.fmEdit.lbBName \
    	-text "Label:"
  grid .tlMenuEdit.fmEdit.lbBName \
    	-row 1 \
	-column 0 \
	-sticky w
  entry .tlMenuEdit.fmEdit.enBNameText \
    	-width 15 \
	-textvariable menu_item_name \
	-background white
  grid .tlMenuEdit.fmEdit.enBNameText \
 	-row 1 \
	-column 1 \
	-sticky w
  label .tlMenuEdit.fmEdit.lbBCommand \
   	-text "Command:"
  grid .tlMenuEdit.fmEdit.lbBCommand \
	-row 2 \
	-column 0 \
	-pady 10
  entry .tlMenuEdit.fmEdit.enBCommandText \
	-textvariable menu_item_command \
	-background white \
	-width 40
  grid .tlMenuEdit.fmEdit.enBCommandText \
	-row 2 \
	-column 1
  checkbutton .tlMenuEdit.fmEdit.btForceToUpper \
  	-text "Force command string to upper case" \
	-highlightthickness 0 \
	-variable MCForceCommandToUpper \
	-onvalue "YES" \
	-offvalue "NO"
  grid .tlMenuEdit.fmEdit.btForceToUpper \
  	-row 3 \
	-column 0 \
	-columnspan 2
  .tlMenuEdit.fmEdit.btForceToUpper select

  frame .tlMenuEdit.fmExit
  pack .tlMenuEdit.fmExit \
	-side top \
	-pady 5
  button .tlMenuEdit.fmExit.btOK \
	-text "OK" \
	-width 12 \
	-command \
	 {
	    # Just pop up a warning if MCNameText is empty
   	    if { [string length [string trim $menu_item_name]] == 0 || \
	         [string length [string trim $menu_item_command]] == 0 } \
   	    {
		tk_messageBox \
	    	    -title Error \
	    	    -type ok \
		    -icon error \
	    	    -parent .tlMenuEdit \
	    	    -message "The Label and Command fields are mandatory."
   	    } \
	    else \
	    {
		MCSaveMenuItem $menu_item_name $menu_item_command \
		               $macro_line $menu_index \
		               $MCForceCommandToUpper
		MCPopulateMenuList $macro_line
		destroy .tlMenuEdit
	    }
	 }
  pack .tlMenuEdit.fmExit.btOK \
  	-side left \
	-padx 10
  button .tlMenuEdit.fmExit.btCancel \
    	-text "Cancel" \
	-width 12 \
	-command { destroy .tlMenuEdit }
  pack .tlMenuEdit.fmExit.btCancel \
    	-side left \
	-padx 10
}


#------------------------------------------------------------------------------#
# MCSaveMenuItem
#------------------------------------------------------------------------------#
proc MCSaveMenuItem { item_name item_command macro_line \
                      menu_index force_command_to_upper } \
{
  global MCMacrosEdit

  # Don't save if name or command string is empty
  if { [string length [string trim $item_name]] == 0 || \
       [string length [string trim $item_command]] == 0 } { return }

  if { [string match $force_command_to_upper "YES"] } \
  {
  	set new_menu_item "$item_name;[string toupper $item_command]"
  } \
  else \
  {
    	set new_menu_item "$item_name;$item_command"
  }

  if { $menu_index == -999 } \
  {
	# This is a new menu item so add it to the end of the list
    	set MCMacrosEdit($macro_line,M) \
	    [lappend MCMacrosEdit($macro_line,M) $new_menu_item]
	set MCMacrosEdit($macro_line,C) \
	    "[llength $MCMacrosEdit($macro_line,M)] Menu Items"
  } \
  else \
  {
	# We are modifying an existing menu item so replace the old with the new
	set MCMacrosEdit($macro_line,M) [lreplace $MCMacrosEdit($macro_line,M) \
	                                          $menu_index \
	                                          $menu_index \
	                                          $new_menu_item]
  }
}


#------------------------------------------------------------------------------#
# MCMoveMenuItem() -- moves the highlighted menu item up or down in the list   #
#------------------------------------------------------------------------------#
proc MCMoveMenuItem { up_or_down macro_line menu_index } \
{
  global MCMacrosEdit
  global MCMenuList
 
  # Return if no button has been selected
  if { $macro_line == -1 || $menu_index == -1 } { return }
  
  # Set new index value depending on the direction that we are moving
  # the menu item.
  if { [string match $up_or_down "UP"] } \
  {
  	if { $menu_index == 0 } { return }
	set new_index [expr $menu_index - 1]
  } \
  elseif { [string match $up_or_down "DOWN"] } \
  {
  	if { $menu_index == [expr [llength $MCMacrosEdit($macro_line,M)] - 1] } \
	{
	    return
	}
	set new_index [expr $menu_index + 1]
  }

  # Save item to be moved
  set item_to_move [lindex $MCMacrosEdit($macro_line,M) $menu_index]

  # Remove item from list
  set MCMacrosEdit($macro_line,M) \
  	[lreplace $MCMacrosEdit($macro_line,M) $menu_index $menu_index]

  # Insert item in new position
  set MCMacrosEdit($macro_line,M) \
  	[linsert $MCMacrosEdit($macro_line,M) $new_index $item_to_move ]

  # Repopulate the list box
  MCPopulateMenuList $macro_line
  
  $MCMenuList selection set $new_index
}


#------------------------------------------------------------------------------#
# MCParseMenuItem - returns the name and command for a given menu item
#		menu_item_list -> list containing all menu items
#		item_index -> list index of desired menu item
#		which_type -> which component of menu item is desired, 'NAME'
#                             or 'COMMAND'
#------------------------------------------------------------------------------#
proc MCParseMenuItem { menu_item_list item_index which_type } \
{
   set menu_item_string [lindex $menu_item_list $item_index]
   set semicolon_index [string first ";" $menu_item_string]
   set menu_item_name \
   	[string range $menu_item_string 0 [expr $semicolon_index - 1]]
   set menu_item_command \
   	[string range $menu_item_string [expr $semicolon_index + 1] end]

   if { [string match $which_type "NAME"] } \
   {
   	return $menu_item_name
   } \
   elseif { [string match $which_type "COMMAND"] } \
   {
   	return $menu_item_command
   }
}


#------------------------------------------------------------------------------#
# SetButtonType - regular button or menu button
#------------------------------------------------------------------------------#
proc SetButtonType { type } \
{
   if { [string match $type "BUTTON"] } \
   {
	# Add the 'Command' label and entry window
	grid .tlMacroEdit.fmEdit.lbBCommand \
    	   -row 3 \
	   -column 0 \
	   -sticky w \
	   -pady 14
	grid .tlMacroEdit.fmEdit.enBCommandText \
    	   -row 3 \
	   -column 1 \
	   -sticky ew
	grid .tlMacroEdit.fmEdit.btForceToUpper \
  	   -row 4 \
	   -column 0 \
	   -columnspan 2
	
	# Set the 'Force to upper case button' to ON
	.tlMacroEdit.fmEdit.btForceToUpper select
	
	# Remove the menu edit button
	grid remove .tlMacroEdit.fmEdit.btEditMenu
   } \
   elseif { [string match "MENU" $type] } \
   {
	# Remove the 'Command' label and entry window
	grid remove .tlMacroEdit.fmEdit.lbBCommand
	grid remove .tlMacroEdit.fmEdit.enBCommandText
	grid remove .tlMacroEdit.fmEdit.btForceToUpper
	grid .tlMacroEdit.fmEdit.btEditMenu \
	   -row 3 \
	   -column 0 \
	   -columnspan 2 \
	   -pady 10
   }
}


#------------------------------------------------------------------------------#
# SaveButtonSettings
#------------------------------------------------------------------------------#
proc SaveButtonSettings { line force_command_to_upper_case } \
{
   global MCMacrosEdit
   global MCMacroNumEdit
   global MCMacroList
   global MCNameText
   global MCCommandText
   global MCTypeText
   global MCPopupText
   global MCMenuList   

   set MCMacrosEdit($line,N) [string trim $MCNameText]
   if { [string match $force_command_to_upper_case "YES"] } \
   {
      set MCMacrosEdit($line,C) [string toupper [string trim $MCCommandText]]
   } \
   else \
   {
      set MCMacrosEdit($line,C) [string trim $MCCommandText]
   }
   set MCMacrosEdit($line,T) [string trim $MCTypeText]
   set MCMacrosEdit($line,P) [string trim $MCPopupText]

   $MCMacroList delete 0 end
   for { set i 0 } { $i < $MCMacroNumEdit } { incr i } \
   {
	if { [string match $MCMacrosEdit($i,T) "MENU"] } \
    	{
	    set command_string ""
    	} \
    	else \
    	{
	    set command_string $MCMacrosEdit($i,C)
    	}
	
   	set toList [format "%-15s %-15s %s" \
                    [string range $MCMacrosEdit($i,N) 0 11] \
                    $MCMacrosEdit($i,T) \
                    $command_string]
	$MCMacroList insert $i $toList
   } 
}


#------------------------------------------------------------------------------#
# MCDeleteMacro() -- deletes the highlighted entry                             #
#------------------------------------------------------------------------------#
proc MCDeleteMacro { line } \
{
  global MCMacros
  global MCMacroList
  global MCMacrosEdit
  global MCMacroNumEdit
  global ListBoxSelections
  
  if {[string compare $line -1]==0} { return }

  #This negates the list-box selection memory
  set ListBoxSelections($MCMacroList) ""

  if { $line < 0 } { return }
  
  for { set i $line } { $i < [expr $MCMacroNumEdit-1] } { incr i } \
  {
    set prev_line [expr $i + 1]
    set MCMacrosEdit($i,N) $MCMacrosEdit($prev_line,N)
    set MCMacrosEdit($i,C) $MCMacrosEdit($prev_line,C)
    set MCMacrosEdit($i,T) $MCMacrosEdit($prev_line,T)
    set MCMacrosEdit($i,P) $MCMacrosEdit($prev_line,P)
    set MCMacrosEdit($i,M) $MCMacrosEdit($prev_line,M)
  }  
  incr MCMacroNumEdit -1  
  
  $MCMacroList delete 0 end
  for { set i 0 } { $i < $MCMacroNumEdit } { incr i } \
  {
    if { [string match $MCMacrosEdit($i,T) "MENU"] } \
    {
	set command_string ""
    } \
    else \
    {
	set command_string $MCMacrosEdit($i,C)
    }
    set toList [format "%-15s %-15s %s" \
                [string range $MCMacrosEdit($i,N) 0 11] \
                $MCMacrosEdit($i,T) \
                $command_string]
    $MCMacroList insert $i $toList
  }
}


#------------------------------------------------------------------------------#
# MCMoveMacro() -- moves the highlighted entry up or down in the list          #
#------------------------------------------------------------------------------#
proc MCMoveMacro { line up_or_down } \
{
  global MCMacros
  global MCMacroList
  global MCMacrosEdit
  global MCMacroNumEdit
  global ListBoxSelections
 
  # Return if no selection has been made in the list box
  if { $line < 0 } { return }
 
  # Set the value for the new index based on the dirction in which
  # we are moving the macro.
  if { [string match $up_or_down "UP"] } \
  {
	# Return if the selected item is already at the top of the list
 	if { $line == 0 } { return }

  	# Set 'new_line' to the line before the current line
  	set new_line [expr $line - 1]
  } \
  elseif { [string match $up_or_down "DOWN"] } \
  {
	# Return if the selected item is already at the bottom of the list
	if { $line >= [expr $MCMacroNumEdit-1] } { return }

  	# Set 'new_line' to the line after the current line
  	set new_line [expr $line + 1]
  }

  #This negates the list-box selection memory
  set ListBoxSelections($MCMacroList) ""

  # Swap the macros
  set MacroMoveN $MCMacrosEdit($new_line,N)
  set MacroMoveC $MCMacrosEdit($new_line,C)
  set MacroMoveT $MCMacrosEdit($new_line,T)
  set MacroMoveP $MCMacrosEdit($new_line,P)
  set MacroMoveM $MCMacrosEdit($new_line,M)
  set MCMacrosEdit($new_line,N) $MCMacrosEdit($line,N)
  set MCMacrosEdit($new_line,C) $MCMacrosEdit($line,C)
  set MCMacrosEdit($new_line,T) $MCMacrosEdit($line,T)
  set MCMacrosEdit($new_line,P) $MCMacrosEdit($line,P)
  set MCMacrosEdit($new_line,M) $MCMacrosEdit($line,M)
  set MCMacrosEdit($line,N) $MacroMoveN
  set MCMacrosEdit($line,C) $MacroMoveC
  set MCMacrosEdit($line,T) $MacroMoveT
  set MCMacrosEdit($line,P) $MacroMoveP
  set MCMacrosEdit($line,M) $MacroMoveM
  
  if { [string match $MCMacrosEdit($line,T) "MENU"] } \
  {
	set command_string ""
  } \
  else \
  {
	set command_string $MCMacrosEdit($line,C)
  }

  set toList1 [format "%-15s %-15s %s" \
               [string range $MCMacrosEdit($line,N) 0 11] \
               $MCMacrosEdit($line,T) \
               $command_string]

  if { [string match $MCMacrosEdit($new_line,T) "MENU"] } \
  {
	set command_string ""
  } \
  else \
  {
	set command_string $MCMacrosEdit($new_line,C)
  }

  set toList2 [format "%-15s %-15s %s" \
               [string range $MCMacrosEdit($new_line,N) 0 11] \
               $MCMacrosEdit($new_line,T) \
               $command_string]
    
  $MCMacroList delete $line
  $MCMacroList insert $line $toList1
  $MCMacroList delete $new_line
  $MCMacroList insert $new_line $toList2
  $MCMacroList selection clear 0 $MCMacroNumEdit
  $MCMacroList selection set $new_line $new_line
}


#------------------------------------------------------------------------------#
# MCPopulateMenuList - fill menu list
#------------------------------------------------------------------------------#
proc MCPopulateMenuList { macro_line } \
{
  global MCMenuList
  global MCMacrosEdit
  
  # Clear the list
  $MCMenuList delete 0 end

  # Now fill the list
  set num_items [llength $MCMacrosEdit($macro_line,M)]

  for { set i 0 } { $i < $num_items  } { incr i } \
  {
  	set menu_item_name \
	    [MCParseMenuItem $MCMacrosEdit($macro_line,M) $i "NAME"]
	set menu_item_command \
	    [MCParseMenuItem $MCMacrosEdit($macro_line,M) $i "COMMAND"]
	set toList [format "%-23s %s" \
	                   [string range $menu_item_name 0 17] \
			   $menu_item_command]
	$MCMenuList insert $i $toList
  }
}


#------------------------------------------------------------------------------#
# MCGetLine() - Determine which line is currently selected in the macro editor #
#------------------------------------------------------------------------------#
proc MCGetLine { } \
{
    global MCMacroList

    set MCline [$MCMacroList curselection]

    if {[string compare $MCline ""]==0} \
    { 
      return -1
    } \
    else \
    { 
      return $MCline 
    }
}


#------------------------------------------------------------------------------#
# MCGetMenuLine() - Determine which line is currently selected in the menu     #
#                   editor                                                     #
#------------------------------------------------------------------------------#
proc MCGetMenuLine { } \
{
    global MCMenuList

    set MCline [$MCMenuList curselection]

    if { [string length [string trim $MCline]] == 0 } \
    { 
	return -1
    } \
    else \
    { 
	return $MCline 
    }
}


#------------------------------------------------------------------------------#
# MCEditMacros() - Creates copies of the Macro list to work with in the editor #
#                  Changes are not commited until the 'OK' button is clicked.  #
#------------------------------------------------------------------------------#
proc MCEditMacros { } \
{
     global MCMacros
     global MCMacroNum
     global MCMacrosEdit
     global MCMacroNumEdit

     for { set i 0 } { $i < $MCMacroNum } { incr i } \
     {
	set MCMacrosEdit($i,N) $MCMacros($i,N)
	set MCMacrosEdit($i,C) $MCMacros($i,C)
	set MCMacrosEdit($i,T) $MCMacros($i,T)
	set MCMacrosEdit($i,P) $MCMacros($i,P)
	set MCMacrosEdit($i,M) $MCMacros($i,M)
     }
     
     set MCMacroNumEdit $MCMacroNum
}


#------------------------------------------------------------------------------#
# MCSaveMacros() - Commits changes made to the Macro list in the editor        #
#------------------------------------------------------------------------------#
proc MCSaveMacros { } \
{
     global MCMacros
     global MCMacroNum
     global MCMacrosEdit
     global MCMacroNumEdit

     for { set i 0 } { $i < $MCMacroNumEdit } { incr i } \
     {
	set MCMacros($i,N) $MCMacrosEdit($i,N)
	set MCMacros($i,C) $MCMacrosEdit($i,C)
	set MCMacros($i,T) $MCMacrosEdit($i,T)
	set MCMacros($i,P) $MCMacrosEdit($i,P)
	set MCMacros($i,M) $MCMacrosEdit($i,M)

	# If this is a menu button use the COMMAND string to store
	# the number of menu items.
	if { [string match $MCMacros($i,T) "MENU"] } \
	{
	    set num_menu_items [llength $MCMacros($i,M)]
	    set MCMacros($i,C) "$num_menu_items (NUMBER OF MENU ITEMS)"
	}
     }
     
     set MCMacroNum $MCMacroNumEdit
}
