#!/bin/false

# Copyright(c) 1998, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: inittblite.tcl,v 1.15 2001/10/11 17:23:32 tsmith Exp $

# generic cursor information frame
#
proc initTBCBox {cbox label variable width} {
  frame $cbox -relief flat

  label $cbox.label -text $label
  #   entry $cbox.data -textvariable $variable -relief groove \
  #      -width $width -state disabled
  entry $cbox.data -textvariable $variable -relief flat \
      -width $width -state disabled

  pack append $cbox \
      $cbox.label	{top} \
      $cbox.data	{bottom}
}

# build cursor line/elem and lat/lon fields
#
proc initTBCursorInfo {cinfo} {
  frame $cinfo -relief flat

  initTBCBox $cinfo.line "Line" cursor_line 6
  initTBCBox $cinfo.elem "Elem" cursor_element 6
  initTBCBox $cinfo.lat "Lat" cursor_latitude 10
  initTBCBox $cinfo.lon "Lon" cursor_longitude 10

  pack append $cinfo \
      $cinfo.line	{left} \
      $cinfo.elem	{left} \
      $cinfo.lon	{right} \
      $cinfo.lat	{right}
}

# keep text of loop start/stop button up-to-date
#
proc setLoopButton {val} {
  global loop_pos_button
  global popup_help_text

  if {$val} {
    set text "Stop"
    set popup_help_text($loop_pos_button) "Stop looping"
  } else {
    set text "Loop"
    set popup_help_text($loop_pos_button) "Start looping"
  }

  $loop_pos_button configure -text $text
}

# turn looping on or off
#
proc toggleLoopButton {} {
  set val [UCpeek looping]

  if {$val} {
    UCpoke looping 0
  } else {
    UCpoke looping 1
  }
}

# count the number of frames in the current loop
#
proc getLoopLength {} {
  set start [expr [ucLocation loop_control] + 1]
  set ucloc $start
  set end [expr $start + 1000]
  while {$ucloc < $end && [UCpeek $ucloc]} {
    incr ucloc
  }

  return [expr $ucloc - $start]
}

# set the length of the loop scale widget
#
proc setLoopPosScaleLen {} {
  global loop_pos_scale
  if { [info exists loop_pos_scale] } \
    {
    $loop_pos_scale configure -to [getLoopLength]
    }
}

# set the location of the slider in the loop scale widget
#
proc setLoopPosition {pos} {
  global loop_pos_scale

  if {[$loop_pos_scale get] != $pos} {
    global doNotSetImageFrame

    set doNotSetImageFrame 1
    $loop_pos_scale set $pos
    if {$pos == 1} {
      setLoopPosScaleLen
    }
  }
}

# update the current frame
#
# called by the loop scale widget when the slider position changes
#
proc setScaleFrame {loopOffset} {
  global doNotSetImageFrame

  if $doNotSetImageFrame {
    set doNotSetImageFrame 0
  } else {
    set fnum [UCpeek [expr [ucLocation loop_control] + $loopOffset]]
    setImageFrame $fnum
  }
}

# build loop scale widget
#
proc initTBLoopPos {looppos} {
  global loop_pos_scale
  global loop_pos_button
  global loop_pos_pos
  global popup_help_text
  global doNotSetImageFrame

  set doNotSetImageFrame 0
  set loop_pos_scale $looppos.scale
  set loop_pos_pos 1

  frame $looppos -relief sunken -borderwidth 1
    button $looppos.left -highlightthickness 0 \
               -bitmap @$env(MCLITE_INSTALL)/lib/giu/larr.xbm \
	       -relief raised \
               -command {set a [getLoopLength]
			 if { [expr $loop_pos_pos -1] == 0 } { set loop_pos_pos $a } \
			 else { incr loop_pos_pos -1 }
                         setScaleFrame $loop_pos_pos 
                         }
    scale $loop_pos_scale -from 1 -to 4 -orient horizontal \
               -variable loop_pos_pos \
               -command setScaleFrame
    button $looppos.right -highlightthickness 0 \
               -bitmap @$env(MCLITE_INSTALL)/lib/giu/rarr.xbm \
	       -relief raised \
               -command { set a [getLoopLength]
			  if { [expr $loop_pos_pos + 1] > $a } { set loop_pos_pos 1 } \
			  else { incr loop_pos_pos 1 }
	                  setScaleFrame $loop_pos_pos 
			}
    pack $looppos.left -side left
    pack $loop_pos_scale -side left -expand 1 -fill x
    pack $looppos.right -side left

  set popup_help_text($loop_pos_scale) "Scroll Through Current Loop"
  setLoopPosScaleLen

  set loop_pos_button $looppos.button
  button $loop_pos_button -text "Loop" -command {toggleLoopButton}
  pack $loop_pos_button -side right
  set popup_help_text($loop_pos_button) {Start looping}
}

proc initTBCurFrame {cur} {
  global popup_help_text LMActiveFrame

  frame $cur -relief sunken -borderwidth 1

  # -- have to create a MENUBUTTON here instead and fill with LOOP/FRAME information
  set wbutton [LMCreateLoopButton $cur "Show Frame" NO]
  
  
#ml  set popup_help_text($wbutton) {Show this frame}
  button $cur.button -text Show -command {pickAndShowFrame}

  pack $cur
}

proc initTBFrameCtl {ctlbox} {
  frame $ctlbox -relief raised -borderwidth 1

  initTBLoopPos $ctlbox.looppos
  initTBCurFrame $ctlbox.frame

  pack append $ctlbox \
	$ctlbox.looppos	{top} \
	$ctlbox.frame	{bottom}
}

proc initTBStateCtl {ctlbox text toggleCmd eraseCmd} {
  global imstate
  global grastate
  global popup_help_text

  set helptext [string tolower $text]

  frame $ctlbox -relief sunken -borderwidth 1

  set state_widget $ctlbox.state
  checkbutton $state_widget -text $text \
	-onvalue ON -offvalue OFF -command $toggleCmd
  if {[string compare $helptext "image"] == 0} {
    $state_widget configure -variable imstate
  } else {
    $state_widget configure -variable grastate
  }

  set popup_help_text($state_widget) "Enable/disable frame $helptext"

  button $ctlbox.erase -text Erase -command $eraseCmd
  set popup_help_text($ctlbox.erase) "Erase frame $helptext"
  pack append $ctlbox \
	$state_widget	{left} \
	$ctlbox.erase	{right}

  return $state_widget
}

proc initTBImGrCtl {toggle} {
  global image_state_widget
  global graphics_state_widget

  frame $toggle -relief sunken -borderwidth 1

  set image_state_widget \
      [initTBStateCtl $toggle.image "Image" "toggleFrameState I" \
	   {catch {exec eg.k I}}]

  set graphics_state_widget \
      [initTBStateCtl $toggle.graphics "Graphics" "toggleFrameState G" \
	   {catch {exec eg.k}}]

  pack append $toggle \
	$toggle.image		{top fillx} \
	$toggle.graphics	{bottom fillx}
}

proc initToolbar {toolbar} {
  frame $toolbar -relief flat -borderwidth 1

  initTBCursorInfo $toolbar.cinfo
  pack $toolbar.cinfo	-side right

  initTBFrameCtl $toolbar.framectl
  pack $toolbar.framectl -side left

#ml  initTBImGrCtl $toolbar.imgrctl
#ml  pack $toolbar.imgrctl -side left
}
